@@ -5,13 +5,13 @@ import torch.nn as nn
 from ray.rllib import SampleBatch
 from ray.rllib.evaluation.postprocessing import compute_advantages
 from ray.rllib.evaluation.postprocessing import Postprocessing
-from ray.rllib.policy.policy import LEARNER_STATS_KEY
 from ray.rllib.utils import override
 from torch.nn.utils import parameters_to_vector
 from torch.nn.utils import vector_to_parameters
 
 from raylab.options import configure
 from raylab.options import option
+from raylab.policy import learner_stats
 from raylab.policy import TorchPolicy
 from raylab.policy.action_dist import WrapStochasticPolicy
 from raylab.torch.optim import build_optimizer
@@ -98,6 +98,7 @@ class TRPOTorchPolicy(TorchPolicy):
         )
         return sample_batch
 
+    @learner_stats
     @override(TorchPolicy)
     def learn_on_batch(self, samples):
         batch_tensors = self.lazy_tensor_dict(samples)
@@ -108,7 +109,7 @@ class TRPOTorchPolicy(TorchPolicy):
         info.update(self.extra_grad_info(batch_tensors))
         info.update(self.get_exploration_info())
 
-        return {LEARNER_STATS_KEY: info}
+        return info
 
     def _update_actor(self, batch_tensors):
         info = {}
